@@ -102,11 +102,84 @@ class Request(object):
         self.content_length = environ.get('CONTENT_LENGTH', '')
         self.uri = URI(environ)
         self.res = Response(self.uri())
-        self.query = cgi.parse_qs(self.uri.query)
-        # should I seek back to 0 after this?
-        self.form = cgi.FieldStorage(fp=environ['wsgi.input'],
-                                     environ=environ,
-                                     keep_blank_values=True) 
+        self._parse_query()
+        self._parse_form()
+
+    def _parse_query(self):
+        """Populate a dictionary in self.query from the querystring.
+        
+        The dictionary created will have strings or lists of strings
+        as its values.
+        
+        If you have a parameter like ?foo[]=bar then 
+        self.query['foo'] == ['bar'] ('bar' will be in a list even if 
+        it has only one member). The key will be 'foo'.
+
+        In other words, this is a way to indicate that you want a 
+        list and avoid boilerplate.
+
+        When a name does not end in brackets, you will end up with a 
+        list only if there is more than one parameter by that name.
+
+        If 'foo[]' and 'foo' are both used as input names, they will 
+        step on each other. Use one or the other.
+        """
+        qu = cgi.parse_qs(self.uri.query)
+        query = {}
+        for key, value in qu.iteritems():
+            if key.endswith('[]'):
+                if not isinstance(value, list):
+                    value = [value]
+                query[key[:-2]] = value
+            else:
+                if isinstance(value, list) and len(value) == 1:
+                    value = value[0]
+                query[key] = value
+        self.query = query
+
+    def _parse_form(self):
+        """Populate a dictionary in self.form from webform data.
+        
+        The dictionary created will have strings, cgi.FieldStorage 
+        objects (used for file uploads) or lists of strings and/or 
+        cgi.FieldStorage objects as its values.
+        
+        If your form data has an field named "foo[]", its value 
+        will be found in self.form in a list even if it has only 
+        one member. The key will be 'foo'.
+
+        In other words, this is a way to indicate that you want a
+        list and avoid boilerplate.
+
+        When a name does not end in brackets, you will end up with 
+        a list only if there is more than one field by that name.
+
+        If 'foo[]' and 'foo' are both used as input names, they 
+        will step on each other. Use one or the other.
+        """
+        fs = cgi.FieldStorage(fp=self.environ['wsgi.input'],
+                              environ=self.environ,
+                              keep_blank_values=True) 
+        form = {}
+        for key in fs:
+            value = fs[key]
+            if key.endswith("[]"):
+                if not isinstance(value, list):
+                    value = [value]
+                key = key[:-2]
+            if isinstance(value, list):
+                newvalue = []
+                for v in value:
+                    if v.filename:
+                        newvalue.append(v)
+                    else:
+                        newvalue.append(v.value)
+                form[key] = newvalue
+            else:
+                if value.filename is None:
+                    value = value.value
+                form[key] = value
+        self.form = form
 
     def redirect(self, location, permanent=True):
         """Set the location header and status."""
@@ -145,15 +218,15 @@ class Yaro(object):
             body = req.res.body
         start_response(req.res.status, req.res._headers)
         if isinstance(body, str):
-            return body
-        elif isiterable(body):
             return [body]
+        elif isiterable(body):
+            return body
         else:
             return util.FileWrapper(body)
 
 
 def isiterable(it):
-    """return True if 'it' is iterable else return False."""
+    """Return True if 'it' is iterable else return False."""
     try:
         iter(it)
     except:
@@ -163,9 +236,10 @@ def isiterable(it):
 
 
 if __name__ == '__main__':
-    from wsgiref.simple_server import make_server
-    from yaro import Yaro
+
     import code
+    
+    from wsgiref.simple_server import make_server
 
     def foo(req):
         #req.res.body = 'Hello, World!'
